#define XK_MISCELLANY
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include "xhbasic.h"
#include "ring.h"
#include "monitor.h"
#include "title1.h"
#include "title2.h"
#include "title3.h"
#include "letter.h"
#include "scorestat.h"
#include "timestat.h"
#include "ringstat.h"
#include "numbers.h"
#include "xhbg1.xpm"
Pixmap background;

char *soundArgs[] = {
  "xhsound",
  "sonjmp.pcm",
  "sonring.pcm",
  "sonlring.pcm",
  "sonhurt.pcm",
  "sonshld.pcm",
  "sonkill.pcm",
  NULL
};

extern char *ZoneTitle;

XhedgehogWin::XhedgehogWin(Display *dpy) :
  SpriteAppWin(dpy,"Xhedgehog version 0.2.0",400,300) {
    pipe1[0] = -1;
    pipe1[1] = -1;
    pipe2[0] = -1;
    pipe2[1] = -1;
    pid = -1;
    InitSoundSrv();
    InitializeLevel();
}

void XhedgehogWin::ProcessEvent(XEvent *evt) {
  KeySym ks;
  switch(evt->type) {
  case KeyPress:
    ks = XLookupKeysym((XKeyEvent *)evt,0);
    switch(ks) {
    case XK_Escape:
      Quit();
      break;
    case XK_Right:
      s->KeyDown(RIGHTKEY);
      break;
    case XK_Left:
      s->KeyDown(LEFTKEY);
      break;
    case XK_Down:
      s->KeyDown(DOWNKEY);
      break;
    case XK_Up:
      s->KeyDown(UPKEY);
      break;
    case XK_space:
      s->KeyDown(JUMPKEY);
      break;
    }
    break;
  case KeyRelease:
    ks = XLookupKeysym((XKeyEvent *)evt,0);
    switch(ks) {
    case XK_Right:
      s->KeyUp(RIGHTKEY);
      break;
    case XK_Left:
      s->KeyUp(LEFTKEY);
      break;
    case XK_Down:
      s->KeyUp(DOWNKEY);
      break;
    case XK_Up:
      s->KeyUp(UPKEY);
      break;
    case XK_space:
      s->KeyUp(JUMPKEY);
      break;
    }
    XSync(display,False);
    break;
  }
}

void XhedgehogWin::InitializeLevel() {
  Pixmap bg;
  LoadPixmap(xhbg1_xpm,&background,&bg);
  XCopyArea(display,background,Backpxm,bufgc,0,0,cx,cy,0,0);
  XFreePixmap(display,background);
  MakeTiles(this);
  MakeSprites(this);
  s = new Sonic(this,6 * TILE_SIZE,34 * TILE_SIZE);
  new ScoreStat(this);
  new TimeStat(this);
  new RingStat(this);
  new NumberSprite(this,60,40,&(s->rings),3);
  new Title1(this);
  new Title2(this);
  new Title3(this,0);
  AnimateString1(this,ZoneTitle);
  s->SetFlag(WALKFLAG);
  //s = CreateCharacter(this,0,0);
  AirJordan = false;
}

void XhedgehogWin::InitSoundSrv() {
  fprintf(stderr,"Initializing sound server...\n");
  pipe(pipe1);
  pipe(pipe2);
  if((pid = fork()) < 0) {
    perror("XhedgehogWin::InitSoundSrv");
  }
  if(pid == 0) {
    close(pipe1[0]);
    dup2(pipe1[1],1);
    close(pipe2[1]);
    dup2(pipe2[0],0);
    if(execvp(soundArgs[0],soundArgs) < 0) {
      perror("XhedgehogWin::InitSoundSrv");
      exit(1);
    }

  }
  else {
    close(pipe1[1]);
    close(pipe2[0]);
    struct sigaction sa;
    sa.sa_handler = SIG_IGN;
    sa.sa_mask = NULL;
    sa.sa_flags = SA_RESTART;
    sigaction(SIGPIPE,&sa,NULL);
  }
}

void XhedgehogWin::SoundSrvPlay(unsigned char s) {
  if(pipe2[1] >= 0) {
    if(write(pipe2[1],&s,1) < 0) {
      perror("XhedgehogWin::SoundSrvPlay");
      fprintf(stderr,"Error writing to sound server; disabling sound.\n");
      close(pipe2[1]);
      close(pipe1[0]);
      pipe1[0] = -1;
      pipe1[1] = -1;
      pipe2[0] = -1;
      pipe2[1] = -1;
    }
  }
}

void XhedgehogWin::KillSoundSrv() {
  if(pid > 0) {
    fprintf(stderr,"Deactivating sound server...\n");
    kill(pid,SIGKILL);
  }
}
